#include "main.h"

//Global variable for holding the problem instance
GRAPH G;

void exitAndLog(string msg) {
	//Outputs the msg to the scrren and log file and then exits
	ofstream resultsLog("resultsLog.txt", ios::app);
	resultsLog << msg;
	resultsLog.close();
	cout << msg;
	exit(1);
}

void usage() {
	cout << "Algorithm for the Maximum Happy Vertices Problem\n\n"
		<< "USAGE:\n\n"
		<< "-----------------\n\n"
		<< "<InputFile>     (Required. File must be in DIMACS format)\n"
		<< "-a <int>        (Algorithm (1/2). 1 = Greedy-MHV (assign all free vertices to same colour, take the best of the k options);\n"
		<< "                                  2 = Growth-MHV (Algorithm described by Li and Zhang).\n"
		<< "                                  Default = 1)\n"
		<< "-r <int>        (Random seed. Default = 1)\n\n"
		<< "-----------------\n";
	exit(0);
}

int main(int argc, char **argv) {

	if (argc <= 1) {
		usage();
	}

	//PARAMETERS----------------------------------------------------------------------------------
	//VARIABLES (DEFAULT VALS)
	int i, seed = 1, algorithm = 1;
	double duration;
	string filename;
	
	try {
		for (i = 1; i < argc; i++) {
			if (strcmp("-r", argv[i]) == 0) {
				seed = atoi(argv[++i]);
			}
			else  if (strcmp("-a", argv[i]) == 0) {
				algorithm = atoi(argv[++i]);
			}
			else {
				cout << "Happy Algorithm using <" << argv[i] << ">\n\n";
				filename = argv[i];
				readInputFile(filename);
			}
		}
	}
	catch (...) {
		cout << "Error with input arguments. Ending...\n";
		usage();
	}
	srand(seed);

	//Get the upper bound using the Lewis-Thiruvady-Morgan method
	int UB = getUpperBound();
	
	//Start timer for the algorithm of choice
	clock_t runStart = clock();

	//Make the solution
	SOL S;
	makeInitSol(S, algorithm);

	//Stop the timer
	duration = (clock() - runStart) / double(CLOCKS_PER_SEC);
	
	//Make some output
	checkSlnValidity(S);
	printSlnToFile(filename, S);
	printRunDetailsToFileAndConsole(filename, S, duration, UB);

	return 0;
}




		




